;**********************************************************************
;   This file is a basic code template for object module code         *
;   generation on the PICmicro PIC16F88. This file contains the       *
;   basic code building blocks to build upon.  As a project minimum   *
;   the 16F88.lkr file will also be required for this file to         *
;   correctly build. The .lkr files are located in the MPLAB          *
;   directory.                                                        *
;                                                                     *
;   If interrupts are not used all code presented between the         *
;   code section "INT_VECTOR and code section "MAIN" can be removed.  *
;   In addition the variable assignments for 'w_temp' and             *
;   'status_temp' can be removed.                                     *
;                                                                     *
;   If interrupts are used, as in this template file, the 16F88.lkr   *
;   file will need to be modified as follows: Remove the lines        *
;     CODEPAGE   NAME=vectors  START=0x0      END=0x4      PROTECTED  *
;   and                                                               *
;     SECTION    NAME=STARTUP  ROM=vectors                            *
;   and change the start address of the page0 section from 0x5 to 0x0 *
;                                                                     *
;   Refer to the MPASM User's Guide for additional information on     *
;   features of the assembler and linker (Document DS33014).          *
;                                                                     *
;   Refer to the respective PICmicro data sheet for additional        *
;   information on the instruction set.                               *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Filename:	    jcmemu.asm                                        *
;    Date:                                                            *
;    File Version:                                                    *
;                                                                     *
;    Author:                                                          *
;    Company:                                                         *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Files required:                                                  *
;                    16F88.lkr                                        *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Notes:                                                           *
;                                                                     *
;                                                                     *
;                                                                     *
;                                                                     *
;**********************************************************************

	list      p=16F88           ; list directive to define processor
	#include <p16F88.inc>        ; processor specific variable definitions

	errorlevel  -302              ; suppress message 302 from list file

;*************************************************************************************************
;		SET CONFIGURTION FLAGS
;*************************************************************************************************
; '__CONFIG' directive is used to embed configuration word within .asm file.
; The lables following the directive are located in the respective .inc file.
; See data sheet for additional information on configuration word settings.

;	CONFIG1
;	-------
;	CP_OFF = Flash Program Code protected
;	CCP1_RB0 = Capture Compare on RB0
;	DEBUG_OFF = In-Circuit Debugger disabled
;	WRT_PROTECT_OFF = Write protection OFF for FLASH memory
;	CPD_OFF = EEPROM Code protection off 
;	LVP_OFF = RB3 digital I/O use HV on MCLR for programmimng
;	BODEN_OFF = Brown out reset enabled
;	MCLR_ON = MCLR is MASTER CLEAR on RA5 (RESET)
;	PWRTE_ON = Power up timer enabled
;	WDT_OFF = Watchdog Timer disabled
;	INTRC_IO = Internal Oscillator Port I/O functions on RA6 and RA7

	__CONFIG    _CONFIG1, _CP_OFF & _CCP1_RB0 & _DEBUG_OFF & _WRT_PROTECT_OFF & _CPD_OFF & _LVP_OFF & _BODEN_ON & _MCLR_OFF & _PWRTE_OFF & _WDT_OFF & _INTRC_IO

;	CONFIG2
;	-------
;	IESO_OFF = Internal Switchover mode disabled
;	FCMEN_OFF = Fail-Safe Clock Monitor disabled

	__CONFIG    _CONFIG2, _IESO_ON & _FCMEN_ON

;***** VARIABLE DEFINITIONS (examples)
; example of using Shared Uninitialized Data Section

INT_VAR		UDATA_SHR	0x71   
w_temp		RES     1		; variable used for context saving 
status_temp	RES     1		; variable used for context saving
pclath_temp	RES		1		; variable used for context saving
temp1		RES		1
temp2		RES		1

;**********************************************************************
STARTUP	CODE
reset_vector:					; processor reset vector
		goto    main            ; go to beginning of program
		nop
		nop
		nop

int_vector:						; interrupt vector location
		nop

PROG0	CODE

;*****************************************************************************
;								IRQ HANDLER
;*****************************************************************************

irq_handler:
		banksel	TMR1H
		bsf		TMR1H, 7			;set bit 15 for 32768 ie 1 second

		bcf		PIR1, TMR1IF		;clear interrupt flag

		banksel	temp1
		movwf	temp1
		btfss	temp1,2
		bsf		temp1,2
		goto	done

		bcf		temp1,2	

done:
		movlw	temp1
		movlw	0x2
		banksel	PORTA
		movwf	PORTA				

		retfie						;return from interrupt

;*****************************************************************************
;								START-UP
;*****************************************************************************

;The program enters here after a Power Down.

main:	
									;configure Output ports

		banksel	PORTA				;select bank or PORTA
		clrf	PORTA				;Initialise PORTA by clearing data latches

		banksel	ANSEL				;select bank of ANSEL (Analogue select)
		clrf	ANSEL				;configure all pins as digital inputs

		movlw	b'01110000'
		movwf	OSCCON				;select 8MHz INTOSC clock
stable:	btfss	OSCCON,IOFS			;INTOSC stable ?
		goto	stable			;no

		banksel	TRISA
		movlw	0x20				;set Data direction for PORTA to all outputs except RA5 (MCLR)
		movwf	TRISA
		
		banksel	PORTB				;select bank or PORTB
		clrf	PORTB				;Initialise PORTB by clearing data latches

		banksel	TRISB
		movlw	0x00				;set Data direction for PORTB 
		movwf	TRISB		

		banksel	OPTION_REG
		movlw	0x00				;enable pull ups on PORTB
		movwf	OPTION_REG
	
		banksel	SPBRG				;BAUD Rate Generator /25
		movlw	0x19				;25
		movwf	SPBRG	

		banksel	TXSTA				;set up transmitter
		movlw	0x26				;8 bit, TX enabled Async, High speed, TSR empty
		movwf	TXSTA

		banksel	RCSTA				;set up receiver
		movlw	0x90				;enable RX port, 8 bit, enable continuous receive
		movwf	RCSTA
		
;		INITIALISE 1 SECOND INTERRUPT ON TIMER1

		banksel	temp1
		clrf	temp1

		banksel	TMR1H
		movlw	0x80				;load count with 8000h = 32768
		movwf	TMR1H
		clrf	TMR1L

		movlw	b'00001111'			;configure for external clock
		movwf	T1CON				;async, external oscillator

		banksel	PIE1
		bsf		PIE1, TMR1IE		;enable Timer 1 interrupt

here:	goto	here

;		Have a go at flashing the LED

		banksel	PORTA
flashd:	movlw	0x2					;RA1 High (LED ON)
		movwf	PORTA				

		call	delay				;delay 76mS			

		movlw	0x00				;RA1 Low (LED OFF)
		movwf	PORTA				

		call	delay				;delay 76mS			

		goto	flashd

;	76mS delay - (3uS*255) * 100))
delay:	movlw	.200
		movwf	temp2
	
loop2:	movlw	.255
		movwf	temp1
loop1:	decfsz	temp1,F				;skip if '0' or 'F'ALSE
		goto	loop1

		decfsz	temp2,F
		goto	loop2
		return


; initialize eeprom locations

DEEPROM	CODE
		DE	0x00, 0x01, 0x02, 0x03

		END                       ; directive 'end of program'

